@@ -151,8 +151,8 @@ class CommandsCache(cabc.Mapping):
     def locate_binary(self, name, ignore_alias=False):
         """Locates an executable on the file system using the cache.
 
-        Arguments
-        ---------
+        Parameters
+        ----------
         name : str
                 name of binary to search for
         ignore_alias : bool, optional
@@ -166,8 +166,8 @@ class CommandsCache(cabc.Mapping):
     def lazy_locate_binary(self, name, ignore_alias=False):
         """Locates an executable in the cache, without checking its validity.
 
-        Arguments
-        ---------
+        Parameters
+        ----------
         name : str
                 name of binary to search for
         ignore_alias : bool, optional
@@ -217,7 +217,14 @@ class CommandsCache(cabc.Mapping):
         """Predicts whether a command list is able to be run on a background
         thread, rather than the main thread.
         """
-        name = self.cached_name(cmd[0])
+        predictor = self.get_predictor_threadable(cmd[0])
+        return predictor(cmd[1:])
+
+    def get_predictor_threadable(self, cmd0):
+        """Return the predictor whether a command list is able to be run on a
+        background thread, rather than the main thread.
+        """
+        name = self.cached_name(cmd0)
         predictors = self.threadable_predictors
         if ON_WINDOWS:
             # On all names (keys) are stored in upper case so instead
@@ -232,15 +239,26 @@ class CommandsCache(cabc.Mapping):
                 if pre in predictors:
                     predictors[name] = predictors[pre]
         if name not in predictors:
-            predictors[name] = self.default_predictor(name, cmd[0])
+            predictors[name] = self.default_predictor(name, cmd0)
         predictor = predictors[name]
-        return predictor(cmd[1:])
+        return predictor
 
     #
     # Background Predictors (as methods)
     #
 
     def default_predictor(self, name, cmd0):
+        """Default predictor, using predictor from original command if the
+        command is an alias, elseif build a predictor based on binary analysis
+        on POSIX, else return predict_true.
+        """
+        # alias stuff
+        if not os.path.isabs(cmd0) and os.sep not in cmd0:
+            alss = getattr(builtins, "aliases", dict())
+            if cmd0 in alss:
+                return self.default_predictor_alias(cmd0)
+
+        # other default stuff
         if ON_POSIX:
             return self.default_predictor_readbin(
                 name, cmd0, timeout=0.1, failure=predict_true
@@ -248,6 +266,28 @@ class CommandsCache(cabc.Mapping):
         else:
             return predict_true
 
+    def default_predictor_alias(self, cmd0):
+        alias_recursion_limit = (
+            10
+        )  # this limit is se to handle infinite loops in aliases definition
+        first_args = []  # contains in reverse order args passed to the aliased command
+        alss = getattr(builtins, "aliases", dict())
+        while cmd0 in alss:
+            alias_name = alss[cmd0]
+            if not isinstance(alias_name, list):
+                return predict_true
+            for arg in alias_name[:0:-1]:
+                first_args.insert(0, arg)
+            if cmd0 == alias_name[0]:
+                # it is a self-alias stop recursion immediatly
+                return predict_true
+            cmd0 = alias_name[0]
+            alias_recursion_limit -= 1
+            if alias_recursion_limit == 0:
+                return predict_true
+        predictor_cmd0 = self.get_predictor_threadable(cmd0)
+        return lambda cmd1: predictor_cmd0(first_args[::-1] + cmd1)
+
     def default_predictor_readbin(self, name, cmd0, timeout, failure):
         """Make a default predictor by
         analyzing the content of the binary. Should only works on POSIX.
@@ -387,6 +427,7 @@ def default_threadable_predictors():
     # alphabetical, for what it is worth.
     predictors = {
         "aurman": predict_false,
+        "asciinema": predict_help_ver,
         "bash": predict_shell,
         "csh": predict_shell,
         "clear": predict_false,
@@ -426,6 +467,7 @@ def default_threadable_predictors():
         "sudo": predict_help_ver,
         "tcsh": predict_shell,
         "telnet": predict_false,
+        "tput": predict_false,
         "top": predict_help_ver,
         "vi": predict_false,
         "view": predict_false,
